from enviroment import env, config
import os
import sys
import random
from tfuncs import InitThread
from tfuncs import PlayThread
from tfuncs import SearchThread
from tfuncs import KeyListenerThread
from tfuncs import LyricsThread
from tfuncs import UpdateThread
from SearchResultList import SearchResultList
from PlayList import PlayList
from DownloadList import DownloadList
from Song import Song
import guihelpers
import pickle
import gtk
import gst
import glib
import gobject
import locale
import gettext
from gettext import gettext as _
from gettext import ngettext as ngettext

try:
    import pynotify
except:
    pass

try:
    import pylast
except:
    pass

class SharkDown:
    """
    The main application class for gSharkDown
    """

    def __init__(self):
        """
        Initialise the main application window
        """
        gobject.threads_init()
        
        self.working = None
        self.playing = None
        # just to prevend scrobbeling more then once for 
        # track.
        self.scrobbled = 0
        # Download dictionary in form Filename : Thread
        self.last_iter = None
        self.windowstate = 1
        self.current_song = None
        self.play_thread = None
        self.notify_timeout_id = None
        
        init_thread = InitThread(self)
        init_thread.start()
        
        # Main window initialization
        builder = gtk.Builder()
        builder.set_translation_domain(env().APP)
        builder.add_from_file('%s/data/main_window.ui' % env().BASEPATH)
        self.window = builder.get_object('mainwindow')
        self.window.connect('delete-event', self.on_window_close)
        self.play_button = builder.get_object('toolbutton_play')
        self.ff_button = builder.get_object('toolbutton_play_next')
        self.prev_button = builder.get_object('toolbutton_play_previous')
        self.lyrics_button = builder.get_object('toolbutton_lyrics')
        self.songinfo = builder.get_object('songinfo')
        self.songinfo.set_text("")
        self.prefsmenu = builder.get_object('mainmenu')
        self.lovebutton = builder.get_object('toolbutton3')
        repeat_button = builder.get_object('togglebutton_repeat')
        repeat_button.set_active(int(config()['repeat_playlist']))
        shuffle_button = builder.get_object('togglebutton_shuffle')
        shuffle_button.set_active(int(config()['shuffle_playlist']))
        self.player = gst.element_factory_make("playbin", "player")

        # Completition configuration
        self.entry = builder.get_object('toolentry_search')
        completition = gtk.EntryCompletion()
        comp_store = gtk.ListStore(str)
        completition.set_model(comp_store)
        for item in config()['completition'].split("|"):
            comp_store.append([item])
        completition.set_minimum_key_length(1)
        completition.set_text_column(0)
        self.entry.set_completion(completition)

        # Results List definition
        self.result = SearchResultList(builder.get_object('list_results'))
        self.result.get_view().connect("row-activated", self.on_result_row_activated)

        # Play List definition
        self.sw2 = builder.get_object('scrolledwindow2')
        self.playlist = PlayList(gtk.IconView())
        self.playlist.get_view().connect('item-activated', self.on_playlist_double_click)
        self.playlist.get_view().show()
        self.sw2.add(self.playlist.get_view())

        # Download list definition
        self.downloads = DownloadList(builder.get_object('list_downloads'))
        self.downloads.connect("downloads-changed", self.on_downloads_changed)
        self.downloads.get_view().hide_all()
        
        # Downloads related initialization
        self.downloads_count = builder.get_object('downloads_count')
        self.downloads_expander = builder.get_object('expander_download')
        self.downmenu = builder.get_object('downloadmenu')
        self.on_downloads_changed(self.downloads, 0)
        
        # Song lists initialization
        if os.path.exists("%s/.gsharkdown/playlist.pkl" % os.environ.get("HOME")):
            self.load_saved_playlist("%s/.gsharkdown/playlist.pkl" % os.environ.get("HOME"))
        else:
            print "Playlist not found"
            
        self.staticon = guihelpers.GsharkIndicator(self)
        
        # Set default directory if is empty
        if config()['down_path'] == "":
            config()['down_path'] = env().get_default_down_path()

        # Pynotify initialization
        if int(config()['show_notification']) == 1 and env().HAVE_NOTIFY:
            pynotify.init("gSharkDown")
            
        # Scrobbling initialization
        self.lastfm = None
        if int(config()['scrobbling']) == 1 and env().HAVE_PYLAST:
            try:
                self.lastfm = pylast.LastFMNetwork(api_key = env().LASTFM_KEY,
                                          api_secret = env().LASTFM_SECRET,
                                          username = config()['lastuser'],
                                          password_hash = config()['lastpass'])
                self.lovebutton.set_sensitive(True)
            except pylast.WSError:
                error = guihelpers.ErrorMessage(self.window,
                            _("Please check your username and password for Last.fm"))
                error.show_all()
                pass

        builder.connect_signals(self)
        self.tlisten = KeyListenerThread(self)
        self.tlisten.start()
        
        # Updates checking
        if config()['update_checked'] == 0:
            self.check_for_updates()
        else:
            if config()['startup_update_check'] == 1:
                self.check_for_updates()
        
        self.window.set_sensitive(False)
        self.window.show_all()
 
        
    def on_init_thread_success(self):
        self.window.set_sensitive(True)
        if os.path.exists("%s/.gsharkdown/downqueue.pkl" % os.environ.get("HOME")):
            self.load_downqueue_list("%s/.gsharkdown/downqueue.pkl" % os.environ.get("HOME"))
            
    def on_init_thread_failed(self):
        error = guihelpers.ErrorMessage(None,
                        "GrooveShark service has probably changed!\n" +
                        "gSharkDown will not function propperly, so please" +
                        "be patient until we find a solution.")
        error.show_all()

    def get_iter_last(self, model):
        """
        Get the last iter from a model
        """
        rows = model.iter_n_children(None);
        return model.get_iter([rows - 1]);

    def result_song(self, index):
        return self.result.get_song(index)

    def playlist_song(self, index):
        return self.playlist.get_song(index)

    def show_prefs_menu(self, button, data = None):
        self.prefsmenu.popup(None, None, None, 0, 0)

    def load_saved_playlist(self, path):
        """
        Loads the saved playlist from .pkl file
        """
        file = open(path, 'rb')
        eoferror = True

        while eoferror:
            try:
                song = Song(pickle.load(file))
                self.playlist.append_song(song)
            except:
                eoferror = False
                
        file.close()
                
    def save_playlist(self, path):
        """
        Saves the playlist from the playlist treeview to 
        /home/$USER/.gsharkdown/playlist.pkl
        """
        try:
            output = open(path, 'w')
            for i in self.playlist.range():
                pickle.dump(self.playlist.get_song(i).get_data(), output)
            output.close()
        except:
            error = guihelpers.ErrorMessage(None,
                            _("Error while saving the playlist"))
            error.show_all()

    def load_downqueue_list(self, path):
        """
        Loads the saved playlist from the .pkl file
        """
        file = open(path, 'rb')
        eoferror = True

        while eoferror:
            try:
                song = Song(pickle.load(file))
                self.downloads.append_song(song)
            except:
                eoferror = False
                
        file.close()

    def save_downqueue(self, path):
        """
        Saves the download queue to pickle file in
        /home/$USER/.gsharkdown/downqueue.pkl
        """
        try:
            output = open(path, 'w')
            for i in self.downloads.range():
                song = self.downloads.get_song(i)
                if song.get_download_progress() < 100:
                    data = self.downloads.get_song(i).get_data()
                    data["filename"] = self.downloads.get_song(i).get_filename()
                    pickle.dump(data, output, -1)
            output.close()
        except Exception, e:
            error = guihelpers.ErrorMessage(None,
                            _("Error while saving the download queue"))
            error.show_all()


    def on_playlist_save_as(self, widget, data = None):
        """
        Saves the playlist to .pkl file in directory
        choosen from the user
        """
        savedlg = gtk.FileChooserDialog(_("Save playlist to file"),
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                       gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        response = savedlg.run()

        if response == gtk.RESPONSE_OK:
            filename = savedlg.get_filename()
            if filename[-4:] != ".pkl":
                filename += ".pkl"
            self.save_playlist(filename)
        savedlg.destroy()

    def on_playlist_open(self, widget, data = None):
        """
        Opens a usersaved playlist from .pkl file
        """
        opendlg = gtk.FileChooserDialog(_("Select Playlist to open"),
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_OPEN,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                       gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        openfilter = gtk.FileFilter()
        openfilter.set_name(_('Playlist files'))
        openfilter.add_pattern("*.pkl")

        opendlg.add_filter(openfilter)

        response = opendlg.run()

        if response == gtk.RESPONSE_OK:
            self.playlist.clear()
            self.stop_play()
            self.load_saved_playlist(opendlg.get_filename())
            self.play_first_song()
        opendlg.destroy()

    def on_window_close(self, widget, data = None):
        """
        Closes the main window
        """
        self.window.hide_on_delete()
        self.windowstate = 0
        return True
    
    def check_for_updates(self):
        """
        Checks if a new version of the application is available
        on the application website. NEVER change the VERSION file
        manually!
        """
        t = UpdateThread()
        t.start()
    
    def on_check_for_updates(self, widget, data = None):
        self.check_for_updates()

    def on_statusicon_clicked(self, widget, data = None):
        """
        Executed on left clicking the statusicon
        """
        if(self.windowstate == 0):
            self.window.show_all()
            self.windowstate = 1
        else:
            self.window.hide_on_delete()
            self.windowstate = 0
            return True

    def on_quit_app(self, widget, data = None):
        """
        Confirmation dialog when exiting the application
        """
        if config()['quit_without_confirmation'] == '0':
            dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL,
                                  type = gtk.MESSAGE_QUESTION,
                                  buttons = gtk.BUTTONS_YES_NO,
                                  message_format = _("Are you sure you want to quit gSharkDown?"))
            dialog.set_title(_("Quit?"))
            response = dialog.run()
            dialog.destroy()
        else:
            response = gtk.RESPONSE_YES

        if response == gtk.RESPONSE_YES:
            self.save_playlist("%s/.gsharkdown/playlist.pkl" % os.environ.get("HOME"))
            self.save_downqueue("%s/.gsharkdown/downqueue.pkl" % os.environ.get("HOME"))
            self.downloads.stop_all_downloads()
            gtk.main_quit()

    def on_copy_song(self, widget, data = None):
        """
        Copy the selected song name from the results list in the
        GNOME Clipboard
        """
        select = self.result.get_selected_rows()
        if len(select) > 0:
            song = self.result.get_song(select[0])
            copystring = "%s - %s" % (song.get_artist(), song.get_title())
            clipboard = gtk.clipboard_get()
            clipboard.set_text(copystring)
            clipboard.store()

    def on_add_to_playlist(self, widget, data = None):
        """
        Adds selected item from the results to the playlist
        """
        select = self.result.get_view().get_selection().get_selected_rows()
        for item in select[1]:
            song = self.result.get_song(item[0])
            self.playlist.append_song(song)

    def on_remove_from_playlist(self, widget, data = None):
        """
        Removes an item from the playlist and the playitems list
        """
        selection = self.playlist.get_selected_rows()
        if len(selection) > 0:
            for i in selection:
                self.playlist.get_model().remove( self.playlist.get_model().get_iter(i) )
                
            if len(self.playlist) > 0:
                select_path = selection[len(selection) - 1]
                if select_path[0] >= len(self.playlist):
                    select_path = (select_path[0]-1,)
                self.playlist.get_view().select_path(select_path)
                
            if self.get_playing_iter() == None:
                self.stop_play()

    def on_clear_playlist(self, widget, data = None):
        """
        Removes all items from the playlist
        """
        self.stop_play()
        self.playlist.clear()

    def on_toggle_repeat(self, widget, data = None):
        """
        Toggles the configuration option for looping trough the
        playlist
        """
        config()['repeat_playlist'] = int(widget.get_active())
        config().write()

    def on_toggle_shuffle(self, widget, data = None):
        """
        Toggle the configuration option for shuffleing the playlist
        """
        config()['shuffle_playlist'] = int(widget.get_active())
        config().write()

    def on_playlist_double_click(self, widget, path, data = None):
        """
        Starts playling on double click event on the playlist
        treeview.
        """
        self.playlist.get_view().unselect_all()
        self.set_playing_song(self.playlist.get_song(path))

    def on_result_row_activated(self, path, column, data = None):
        """
        On double click in the search treeview the item is added to 
        the playlist and starts playling.
        """
        song = self.result.get_song(column[0])
        self.playlist.append_song(song)
            
    def get_playing_iter(self):
        return self.playlist.get_song_iter(self.get_playing_song())
    
    def get_playing_song(self):
        return self.playlist.get_playing_song()
        
    def set_playing_song(self, song):
        """
        Updates some labels and icons when a song is playing
        """
        self.playlist.set_playing_song(song)
        
        if song != None:
            self.staticon.change_status_playing()
            
            play_stop_image = gtk.Image()
            play_stop_image.set_from_stock(gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_MENU)
            self.staticon.change_playbutton(play_stop_image)
            self.play_button.set_stock_id(gtk.STOCK_MEDIA_STOP)
            self.scrobbled = 0

            self.songinfo.set_markup(
                _("<b>Playing:</b> {artist} - {title}").format(artist = glib.markup_escape_text(song.get_artist()),
                                         title = glib.markup_escape_text(song.get_title()))
            )
            if int(config()['show_notification']) == 1 and env().HAVE_NOTIFY:
                if self.notify_timeout_id != None:
                    gobject.source_remove(self.notify_timeout_id)
                self.notify_timeout_id = gobject.timeout_add(1000, self.show_playing_song_notification)
                
            self.player.set_state(gst.STATE_NULL)
            
            if self.play_thread != None:
                # Avoid the current song streaming
                self.play_thread.stop()
                self.play_thread.join()
            self.play_thread = PlayThread(self, song)
            self.play_thread.start()
        else:
            if self.play_thread != None:
                self.play_thread.stop()
            
            self.staticon.change_status_stopped()
            play_stop_image = gtk.Image()
            play_stop_image.set_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_MENU)
            self.staticon.change_playbutton(play_stop_image)
            self.player.set_state(gst.STATE_NULL)
            self.play_button.set_stock_id(gtk.STOCK_MEDIA_PLAY)
            self.songinfo.set_markup("")
    
    def show_playing_song_notification(self):
        song = self.get_playing_song()
        n = pynotify.Notification(_("Now playing"),
            "%s - %s" % (glib.markup_escape_text(song.get_artist()),
                         glib.markup_escape_text(song.get_title())),
            "audio-x-generic")
        n.set_timeout(50)
        n.show()
    
    def play_first_song(self):
        if len(self.playlist) > 0:
            self.set_playing_song(self.playlist.get_song(0))
    
    def on_play_selected(self, widget, data = None, row = None):
        """
        Starts the play thread
        """
        if self.get_playing_song() == None:
            self.play_first_song()
        else:
            self.stop_play()
            
    def on_play_next(self, widget, data = None):
        self.play_next()
        
    def on_play_previous(self, widget, data = None):
        self.play_previous()
    
    def next_shuffled_index(self):
        path = None
        
        if len(self.playlist) == 1:
            return 0
        
        if self.get_playing_song() != None:
            path = self.playlist.get_song_path(self.get_playing_song())
            
        while True:
            index = random.randint(0, len(self.playlist) - 1)
            if path == None or index != path[0]:
                break
            
        return index
    
    def play_next(self):
        """
        Plays the next item from the playlist
        """
        if len(self.playlist) == 0:
            return
        
        if self.get_playing_song() == None:
            self.play_first_song()
        else:
            if int(config()['shuffle_playlist']) == 1:
                index = self.next_shuffled_index()
            else:
                index = self.playlist.get_song_path(self.get_playing_song())[0] + 1

            if index >= len(self.playlist):
                if int(config()['repeat_playlist']) == 1:
                    index = 0
                else:
                    index = None
                    self.stop_play()
            
            if index != None:
                self.set_playing_song( self.playlist.get_song((index,)) )
        
    def play_previous(self):
        """
        Plays the previous song from the playlist
        """
        if len(self.playlist) == 0:
            return
        
        if self.get_playing_iter() == None:
            self.play_first_song()
        else:
            if int(config()['shuffle_playlist']) == 1:
                index = self.next_shuffled_index()
            else:
                index = self.playlist.get_song_path(self.get_playing_song())[0] - 1

            if index < 0:
                if int(config()['repeat_playlist']) == 1:
                    index = len(self.playlist) - 1
                else:
                    index = 0
            
            self.set_playing_song( self.playlist.get_song((index,)) )

    def stop_play(self):
        """
        Stop current playing
        """
        self.set_playing_song(None)

    def on_volume_change(self, widget, data = 0.5):
        """
        Volume change handler
        """
        self.player.set_property("volume", float(data))
        return True

    def on_show_info(self, widget, data = None):
        """
        Song infromation dialog for first item selected from the search results.
        """
        select = self.result.get_view().get_selection().get_selected_rows()
        if self.result.get_view().get_selection().count_selected_rows() > 0:
            song = self.result.get_song(select[1][0])
            dialog = guihelpers.SongInfoDialog(song)
            dialog.show_all()
            

    def on_show_lyrics(self, widget, data = None):
        """
        Tries to retrieve lyrics for selected song from the playlist.
        """
        if self.get_playing_song() != None:
            t = LyricsThread(self, self.get_playing_song())
            t.start()
        else:
            info = guihelpers.InfoDialog(self.window,
                _("There should be a playing song to view\nthe lyrics for it."))
            info.show_all()

    def on_show_about(self, widget, data = None):
        """
        About dialog for the application
        """
        guihelpers.AboutDialog(self)

    def on_open_preferences(self, widget, data = None):
        """
        Preferences dialog with save and close buttons
        """
        dialog = guihelpers.PreferencesDialog(self)
        dialog.show_all()
        
    def on_search_grooveshark(self, widget, data = None):
        """
        Starts the search thread
        """
        if widget.get_text() != "":
            if widget.get_text() in config()['completition'].split("|"):
                pass
            else:
                compl = config()['completition'] + widget.get_text() + "|"
                config()['completition'] = compl
                config().write()
            search_thread = SearchThread(self, widget.get_text(), "Songs")
            search_thread.start()

    def get_overwritten_filename(self, filename):
        """
        Gets the file name depending on whether user want to overwrite the file,
        not overwrite it or not save it. Returns the same filename if user want
        to overwrite, the renamed filename or None if the user cancel Download.
        """
        if os.path.exists(filename) == True:
            dialog = gtk.Dialog(_("Overwrite downloaded file?"),
                         self.window,
                         gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                         (_("Cancel download"), gtk.RESPONSE_NONE,
                          _("Download and overwrite"), 0,
                          _("Download renamed"), 1))

            label = gtk.Label(_("The file '%s' already exists, maybe you are downloading a file that you downloaded before.\n\nWhat do you want to do?") % filename)
            label.set_line_wrap(True)
            dialog.vbox.pack_start(label)
            label.show()

            dialog.set_default_response(gtk.RESPONSE_NONE)
            dialog.set_resizable(False)
            dialog.set_size_request(600, 175)
            response = dialog.run()
            dialog.destroy()
        else:
            response = 0

        if response == 1:
            i = 1
            newfilename = ""
            while i == 1 or os.path.exists(newfilename):
                i += 1
                split = os.path.splitext(filename)
                newfilename = split[0] + " (%d)" % i + split[1]

            return newfilename
        elif response == 0:
            return filename
        else:
            return None

    def on_download_selected(self, widget, data = None):
        """
        Starts the download thread
        """
        select = self.result.get_selected_rows()
        for path in select:
            song = self.result.get_song(path)
            
            filename = self.get_overwritten_filename(song.get_default_filename())

            if filename != None:
                song.set_filename(filename)
                self.downloads.append_song_restarting(song)

    def on_downloads_changed(self, widget, count):
        """
        Update the label wich indicates the downloads count.
        """
        text = ngettext("Downloading %d file", "Downloading %d files", count) % count
        self.downloads_count.set_label(text)

    def right_click_download(self, treeview, event):
        """
        Executed when the user rightclicks on selected items in the download list.
        Can handle single and multiple selection.
        """
        if event.button == 3:
            path = treeview.get_path_at_pos(int(event.x), int(event.y))
            selection = treeview.get_selection()
            rows = selection.get_selected_rows()
            try:
                if path[0] not in rows[1]:
                    # Unselect if the user click the right button outside 
                    # the selected items - empty field.
                    selection.unselect_all()
                    selection.select_path(path[0])
            except TypeError:
                pass

            # still not sure about this. The point is that there are many rows selected
            # when you click the right mouse button, but then on_cancel_download still 
            # has no idea what is selected.
            if selection.count_selected_rows() > 1:
                self.downmenu.popup(None, None, None, event.button, event.time)
            else:
                self.downmenu.popup(None, None, None, event.button, event.time)

    def on_cancel_all_downloads(self, widget, data = None):
        """
        Cancel all downloads
        """
        iter = self.downloads.first()
        while iter != None:
            cur = iter
            iter = self.downloads.next(iter)
            self.downloads.get_song(cur).cancel_download()
            
    def on_stop_all_downloads(self, widget, data = None):
        """
        Cancel all downloads
        """
        iter = self.downloads.first()
        while iter != None:
            self.downloads.get_song(iter).pause_download()
            iter = self.downloads.next(iter)
            
    def on_resume_all_downloads(self, widget, data = None):
        """
        Cancel all downloads
        """
        iter = self.downloads.first()
        while iter != None:
            self.downloads.get_song(iter).resume_download()
            iter = self.downloads.next(iter)

    def on_cancel_download(self, menu, data = None):
        """
        Cancel the download. Have to check how to cancel the thread.
        """
        select = self.downloads.get_selected_rows()
        for path in select:
            song = self.downloads.get_song(path).cancel_download()

    def on_stop_download(self, button, data = None):
        """
        TODO: Not yet implemented, have to see how the download
        list will be saved.
        Stop download, in order to be resumed later
        """
        select = self.downloads.get_selected_rows()
        for path in select:
            self.downloads.get_song(path).pause_download()

    def on_resume_download(self, button, data = None):
        """
        TODO: Not yet implemnted, after stopping download we have to
        load the download list in order to resume.
        Resume stopped download
        """
        select = self.downloads.get_selected_rows()
        for path in select:
            self.downloads.get_song(path).resume_download()

    def on_love_song(self, button):
        if self.get_playing_song() == None:
            info = guihelpers.InfoDialog(self.window,
                    _("There is no song to be loved.\n A song should be playing"))
            info.show_all()
        else:
            try:
                track = self.lastfm.get_track(self.get_playing_song().get_artist(),
                                          self.get_playing_song().get_title())
                track.love()
            except pylast.MalformedResponseError, e:
                pass
